/**************************************************************************************
Author:			David Szybala
Program:		Wire_Parameters.ms
Date:			09/03/12
Description:	This script will wire the selected objects controller property to the
desired master controller property.
***************************************************************************************/

try (destroyDialog wireRollout) catch()

(--put script into local scope
	global wireRollout
	local wireIniFile = getDir #plugcfg + "\\QuickWire.ini"
	local wireRolloutPos
	
	/**************************************************************************************/
	fn Get_Controller_Test currentObj transformIndex =
	(
		returnCheck = false
		
		case transformIndex of
		(
			(1):if (classOf currentObj.pos.controller) == Position_XYZ do returnCheck = true
			(2):if (classOf currentObj.pos.controller) == Position_XYZ do returnCheck = true
			(3):if (classOf currentObj.pos.controller) == Position_XYZ do returnCheck = true
			(4):if (classOf currentObj.rotation.controller) == Euler_XYZ do returnCheck = true
			(5):if (classOf currentObj.rotation.controller) == Euler_XYZ do returnCheck = true
			(6):if (classOf currentObj.rotation.controller) == Euler_XYZ do returnCheck = true
			(7):if (classOf currentObj.scale.controller) == ScaleXYZ do returnCheck = true
			(8):if (classOf currentObj.scale.controller) == ScaleXYZ do returnCheck = true
			(9):if (classOf currentObj.scale.controller) == ScaleXYZ do returnCheck = true
		)--end case
		
		returnCheck --return value
	)--end fn
	
	/**************************************************************************************/
	fn Get_Controller currentObj transformIndex =
	(
		returnValue = undefined
		
		case transformIndex of
		(
			(1):returnValue = currentObj.pos.controller[#X_Position]
			(2):returnValue = currentObj.pos.controller[#Y_Position]
			(3):returnValue = currentObj.pos.controller[#Z_Position]
			(4):returnValue = currentObj.rotation.controller[#X_Rotation]
			(5):returnValue = currentObj.rotation.controller[#Y_Rotation]
			(6):returnValue = currentObj.rotation.controller[#Z_Rotation]
			(7):returnValue = currentObj.scale.controller[#X_Scale]
			(8):returnValue = currentObj.scale.controller[#Y_Scale]
			(9):returnValue = currentObj.scale.controller[#Z_Scale]
		)--end case
		
		returnValue --return value
	)--end fn
	
	/**************************************************************************************/
	fn Set_Controller currentObj transformIndex =
	(
		case transformIndex of
		(
			(1):currentObj.pos.controller = Position_XYZ()
			(2):currentObj.pos.controller = Position_XYZ()
			(3):currentObj.pos.controller = Position_XYZ()
			(4):currentObj.rotation.controller = Euler_XYZ()
			(5):currentObj.rotation.controller = Euler_XYZ()
			(6):currentObj.rotation.controller = Euler_XYZ()
			(7):currentObj.scale.controller = ScaleXYZ()
			(8):currentObj.scale.controller = ScaleXYZ()
			(9):currentObj.scale.controller = ScaleXYZ()
		)--end case
	)--end fn
	
	/**************************************************************************************/
	fn Get_Parameter_String transformIndex =
	(
		local returnString = ""
		
		case transformIndex of
		(
			(1):returnString = "X_Position"
			(2):returnString = "Y_Position"
			(3):returnString = "Z_Position"
			(4):returnString = "X_Rotation"
			(5):returnString = "Y_Rotation"
			(6):returnString = "Z_Rotation"
			(7):returnString = "X_Scale"
			(8):returnString = "Y_Scale"
			(9):returnString = "Z_Scale"
		)--end case
		
		returnString --return value
	)--end fn
	
	/**************************************************************************************/
	fn Get_Expression masterController slaveController =
	(
		local expressionArray = #("","","","","","","","") --operation,param,operation,value,operation,value,operation,custom text
		local returnString = ""
		
		expressionArray[2] = Get_Parameter_String wireRollout.masterDropList.selection 
		if wireRollout.masterDropList.selection > 3 and wireRollout.masterDropList.selection < 7 do
		(
			if wireRollout.slaveDropList.selection < 4 or wireRollout.slaveDropList.selection > 6 do
				expressionArray[2] = ("(radToDeg(" + expressionArray[2] + "))")
		)--end if
		
		if wireRollout.offsetCheck.checked do
		(
			if masterController.value > slaveController.value then
			(
				expressionArray[4] = (masterController.value as string)
				expressionArray[6] = (slaveController.value as string)
				expressionArray[3] = expressionArray[5] = " - "
			)--end if
			else
			(
				expressionArray[4] = (slaveController.value as string)
				expressionArray[6] = (masterController.value as string)
				expressionArray[3] = " + "
				expressionArray[5] = " - "
			)--end else
			
			if wireRollout.masterDropList.selection > 3 and wireRollout.masterDropList.selection < 7 do
			(
				if wireRollout.slaveDropList.selection > 3 and wireRollout.slaveDropList.selection < 7 do
				(
					expressionArray[4] = ("degToRad(" + expressionArray[4] + ")")
					expressionArray[6] = ("degToRad(" + expressionArray[6] + ")")
				)--end if
			)--end if
		)--end if
			
		if wireRollout.invertCheck.checked do
		(
			expressionArray[1] = " - "	
			case expressionArray[3] of
			(
				(" - "):expressionArray[3] = " + "
				(" + "):expressionArray[3] = " - "
			)--end case
			case expressionArray[5] of
			(
				(" - "):expressionArray[5] = " + "
				(" + "):expressionArray[5] = " - "
			)--end case
		)--end if
		
		if wireRollout.customText.text != "" do
		(
			expressionArray[7] = (" " + wireRollout.operationList.selected + " ")
			expressionArray[8] = wireRollout.customText.text
		)--end if
		
		returnString = expressionArray[1] + expressionArray[2]
		if wireRollout.offsetCheck.checked do returnString = returnString + expressionArray[3] + "(" + expressionArray[4] + expressionArray[5] + \
					   										 expressionArray[6] + ")"
		if wireRollout.customText.text != "" do returnString = "(" + returnString + ")" + expressionArray[7] + expressionArray[8]
		
		returnString --return value
	)--end fn

	/**************************************************************************************/
	rollout wireRollout "Wire Parameters (XYZ Only)"
	(
		pickButton masterPick "Pick Master" pos:[5,5] width:215 height:20
		label slaveLabel "Slave(s)" pos:[5,30] width:50 height:15
		label masterLabel "Master" pos:[120,30] width:50 height:15
		dropdownlist slaveDropList "" pos:[5,45] width:100 items:#("X Position","Y Position","Z Position","X Rotation","Y Rotation","Z Rotation","X Scale","Y Scale","Z Scale")
		dropdownlist masterDropList "" pos:[120,45] width:100 items:#("X Position","Y Position","Z Position","X Rotation","Y Rotation","Z Rotation","X Scale","Y Scale","Z Scale") enabled:false
		groupbox optionsGrp "Options" pos:[5,70] width:215 height:105
			checkbox offsetCheck "Keep Offset" pos:[15,90] width:80 height:15 enabled:false
			checkbox invertCheck "Invert" pos:[120,90] width:80 height:15 enabled:false
			checkbox autoControlCheck "Assign Controllers" pos:[15,110] width:100 height:15 enabled:false
			label customLabel "Custom String:" pos:[15,132] width:200 height:15
			dropdownlist operationList "" pos:[15,147] width:40 items:#("+","-","*","/") enabled:false
			editText customText "" pos:[58,147] width:155 height:20 readOnly:true
		button addWireButton "Add Wire" pos:[5,180] width:100 height:20 enabled:false
		button removeWireButton "Remove Wire" pos:[120,180] width:100 height:20
		
		-------------------------------------------------------------------------
		on masterPick picked currentObj do
		(
			if currentObj != undefined do
			(
				masterPick.text = currentObj.name
				masterDropList.enabled = offsetCheck.enabled = invertCheck.enabled = \
				autoControlCheck.enabled = operationList.enabled = addWireButton.enabled = true
				customText.readOnly = false
			)--end if
		)--end on
		
		-------------------------------------------------------------------------
		on addWireButton pressed do
		(
			local controllerTest = undefined
			local errors = 0
			local wireExpression = ""
			local masterSubAnim = undefined
			local slaveSubAnim = undefined
			
			deselectNode masterPick.object --in case master node is selected
			
			if autoControlCheck.checked do
			(
				controllerTest = Get_Controller_Test masterPick.object masterDropList.selection
				if controllerTest == false do Set_Controller masterPick.object masterDropList.selection
				
				for current in selection do
				(
					controllerTest = Get_Controller_Test current slaveDropList.selection
					if controllerTest == false do Set_Controller current slaveDropList.selection
				)--end for
			)--end if
			
			controllerTest = Get_Controller_Test masterPick.object masterDropList.selection
			
			if controllerTest then
			(
				masterSubAnim = Get_Controller masterPick.object masterDropList.selection
							
				for current in selection do
				(
					controllerTest = Get_Controller_Test current slaveDropList.selection
					
					if controllerTest then
					(
						slaveSubAnim = Get_Controller current slaveDropList.selection
						
						wireExpression = Get_Expression masterSubAnim slaveSubAnim
						
						paramWire.connect masterSubAnim slaveSubAnim wireExpression
					)--end if
					else
					(
						print ("ERROR: Object " + current.name + " must have controller Position_XYZ, Euler_XYZ or ScaleXYZ.")
						errors += 1
					)--end else
				)--end for
				
				print ("Parameter wiring added. " + (errors as string) + " error(s).")
			)--end if
			else
				print ("ERROR: Master object must have controller Position_XYZ, Euler_XYZ or ScaleXYZ.")
		)--end on
		
		-------------------------------------------------------------------------
		on removeWireButton pressed do
		(
			local storeTrans = undefined
			local controllerTest = false
			local errors = 0
			
			for current in selection do
			(
				controllerTest = Get_Controller_Test current slaveDropList.selection
				
				if controllerTest then
				(
					storeTrans = current.transform
					
					case slaveDropList.selection of
					(
						(1):paramWire.disconnect current.pos.controller[1].controller
						(2):paramWire.disconnect current.pos.controller[2].controller
						(3):paramWire.disconnect current.pos.controller[3].controller
						(4):paramWire.disconnect current.rotation.controller[1].controller
						(5):paramWire.disconnect current.rotation.controller[2].controller
						(6):paramWire.disconnect current.rotation.controller[3].controller
						(7):paramWire.disconnect current.scale.controller[1].controller
						(8):paramWire.disconnect current.scale.controller[2].controller
						(9):paramWire.disconnect current.scale.controller[3].controller
					)--end case
					
					current.transform = storeTrans
				)--end if
				else
				(
					print ("ERROR: Object " + current.name + " must have controller Position_XYZ, Euler_XYZ or ScaleXYZ.")
					errors += 1
				)--end else
			)--for current
			
			print ("Parameter wiring removed. " + (errors as string) + " error(s).")
		)--end on removeWireButton
	)--end rollout
	wireRolloutPos = execute (getIniSetting wireIniFile "Dialog" "Position")
	if wireRolloutPos == OK do wireRolloutPos = [100,100]
	if wireRolloutPos.x > sysinfo.desktopsize.x - 225 do wireRolloutPos.x = sysinfo.desktopsize.x - 230
	if wireRolloutPos.y > sysinfo.desktopsize.y - 265 do wireRolloutPos.y = sysinfo.desktopsize.y - 265
	if wireRolloutPos.x < 0 do wireRolloutPos.x = 0 
	if wireRolloutPos.y < 0 do wireRolloutPos.y = 0
	createDialog wireRollout 225 205 wireRolloutPos.x wireRolloutPos.y
)--end local scope